# -*- coding: utf-8 -*-
"""Test configuration for pytest"""

import pytest
from tests.fixtures.mock_session import mock_session, make_mock_response
from tests.fixtures.mock_streaming_client import mock_streaming_client, mock_generator



# -*- coding: utf-8 -*-
"""Models"""

from .customvocabulary import CustomVocabulary
from .streaming import MediaConfig
from .asynchronous import Job, JobStatus, Account, Transcript, Monologue, Element, CaptionType, \
    SpeakerName
from .insights import TopicExtractionJob, TopicExtractionResult, Topic, Informant, \
    SentimentAnalysisResult, SentimentValue, SentimentMessage, SentimentAnalysisJob
from .language_id import LanguageIdentificationJob, LanguageIdentificationResult, LanguageConfidence
from .customer_url_data import CustomerUrlData



# -*- coding: utf-8 -*-
"""Unit tests for RevAiApiClient"""

import pytest
import re
import json
from requests.exceptions import HTTPError
from src.rev_ai.apiclient import RevAiAPIClient
from src.rev_ai import __version__
from src.rev_ai.baseclient import BaseClient
from tests.helpers.errors import get_error_test_cases

TOKEN = "token"


class TestBaseClient:
    def test_constructor_with_success(self):
        client = BaseClient(TOKEN)

        headers = client.default_headers

        assert headers.get(
            'User-Agent') == 'RevAi-PythonSDK/{}'.format(__version__)
        assert headers.get('Authorization') == 'Bearer token'

    @pytest.mark.parametrize('token', [None, ''])
    def test_constructor_with_no_token(self, token):
        with pytest.raises(ValueError, match='access_token must be provided'):
            RevAiAPIClient(token)

    @pytest.mark.parametrize('error', get_error_test_cases(
        ['invalid-parameters', 'unauthorized', 'forbidden', 'job-not-found', 'invalid-job-state']))
    @pytest.mark.parametrize('method', ["POST", "GET", "DELETE"])
    def test_make_http_request(self, error, method, mock_session,
                               make_mock_response):
        status = error.get('status')
        URL = RevAiAPIClient.base_url
        response = make_mock_response(url=URL, status=status, json_data=error)
        mock_session.request.return_value = response
        client = RevAiAPIClient(TOKEN)

        with pytest.raises(
            HTTPError,
            match="(?=.*{})(?=.*{})".format(
                status,
                re.escape(json.dumps(error).replace('\"', '\'')))
        ):
            client._make_http_request(method, URL)
        mock_session.request.assert_called_once_with(
            method, URL,
            headers=client.default_headers
        )



# -*- coding: utf-8 -*-
"""Top-level package for rev_ai"""

__version__ = '2.18.0'

from .models import Job, JobStatus, Account, Transcript, Monologue, Element, MediaConfig, \
    CaptionType, CustomVocabulary, TopicExtractionJob, TopicExtractionResult, Topic, Informant, \
    SpeakerName, LanguageIdentificationJob, LanguageIdentificationResult, LanguageConfidence, \
    SentimentAnalysisResult, SentimentValue, SentimentMessage, SentimentAnalysisJob, CustomerUrlData



# -*- coding: utf-8 -*-
"""StreamingClient tool used for streaming services"""

import websocket
import threading
import six
import json
from . import __version__

try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode


def on_error(error):
    raise error


def on_close(code, reason):
    print("Connection Closed. Code : {}; Reason : {}".format(code, reason))


def on_connected(job_id):
    print('Connected, Job ID : {}'.format(job_id))


class RevAiStreamingClient:
    def __init__(self,
                 access_token,
                 config,
                 version='v1',
                 on_error=on_error,
                 on_close=on_close,
                 on_connected=on_connected):
        """Constructor for Streaming Client
        :param access_token: access token which authorizes all requests and
            links them to your account. Generated on the settings page of your
            account dashboard on Rev AI.
        :param config: a MediaConfig object containing audio information.
            See MediaConfig.py for more information
        :param version (optional): version of the streaming api to be used
        :param on_error (optional): function to be called when receiving an
            error from the server
        :param on_close (optional): function to be called when the websocket
            closes
        :param on_connected (optional): function to be called when the websocket
            and thread starts successfully
        """
        if not access_token:
            raise ValueError('access_token must be provided')

        if not config:
            raise ValueError('config must be provided')

        self.access_token = access_token
        self.config = config
        self.base_url = 'wss://api.rev.ai/speechtotext/{}/stream'. \
            format(version)
        self.on_error = on_error
        self.on_close = on_close
        self.on_connected = on_connected
        self.client = websocket.WebSocket(enable_multithread=True)

    def start(self,
              generator,
              metadata=None,
              custom_vocabulary_id=None,
              filter_profanity=None,
              remove_disfluencies=None,
              delete_after_seconds=None,
              detailed_partials=None,
              start_ts=None,
              transcriber=None,
              language=None,
              skip_postprocessing=None):
        """Function to connect the websocket to the URL and start the response
            thread
        :param generator: generator object that yields binary audio data
        :param metadata: metadata to be attached to streaming job
        :param custom_vocabulary_id: id of custom vocabulary to be used with this streaming job
        :param filter_profanity: whether to mask profane words
        :param remove_disfluencies: whether to exclude filler words like "uh"
        :param delete_after_seconds: number of seconds after job completion when job is auto-deleted
        :param detailed_partials: whether to receive timestamps and confidence scores
        :param start_ts: number of seconds to offset all hypotheses timings
        :param transcriber: type of transcriber to use to transcribe the media file
        :param language: language to use for the streaming job
        :param skip_postprocessing: skip all text postprocessing on final hypotheses
        """
        url = self.base_url + '?' + urlencode({
            'access_token': self.access_token,
            'content_type': self.config.get_content_type_string(),
            'user_agent': 'RevAi-PythonSDK/{}'.format(__version__)
        })

        if custom_vocabulary_id:
            url += '&' + urlencode({'custom_vocabulary_id': custom_vocabulary_id})

        if metadata:
            url += '&' + urlencode({'metadata': metadata})

        if filter_profanity:
            url += '&' + urlencode({'filter_profanity': 'true'})

        if remove_disfluencies:
            url += '&' + urlencode({'remove_disfluencies': 'true'})

        if delete_after_seconds is not None:
            url += '&' + urlencode({'delete_after_seconds': delete_after_seconds})

        if detailed_partials:
            url += '&' + urlencode({'detailed_partials': 'true'})

        if start_ts:
            url += '&' + urlencode({'start_ts': start_ts})

        if transcriber:
            url += '&' + urlencode({'transcriber': transcriber})

        if language:
            url += '&' + urlencode({'language': language})

        if skip_postprocessing:
            url += '&' + urlencode({'skip_postprocessing': 'true'})

        try:
            self.client.connect(url)
        except Exception as e:
            self.on_error(e)

        self._start_send_data_thread(generator)

        return self._get_response_generator()

    def end(self):
        """Function to end the streaming service, close the websocket.
        """
        self.client.abort()

    def _start_send_data_thread(self, generator):
        """Function to send binary audio data from a generator with threading
        :param generator: generator object that yields binary audio data
        """
        if not generator:
            raise ValueError('generator must be provided')

        if hasattr(self, 'request_thread'):
            if self.request_thread.isAlive():
                raise RuntimeError("""Data is still being sent and will interfere
                    with the responses.""")

        self.request_thread = threading.Thread(
            target=self._send_data,
            args=[generator]
        )
        self.request_thread.start()

    def _send_data(self, generator):
        """Function used in a thread to send requests to the server.
        :param generator: enumerator object that yields binary audio data
        """
        if not generator:
            raise ValueError('generator must be provided')

        for chunk in generator:
            self.client.send_binary(chunk)

        self.client.send("EOS")

    def _get_response_generator(self):
        """A generator of responses from the server. Yields the data decoded.
        """
        while True:
            with self.client.readlock:
                opcode, data = self.client.recv_data()
            if opcode == websocket.ABNF.OPCODE_TEXT:
                if six.PY3:
                    data = data.decode('utf-8')
                data_dict = json.loads(data)
                if data_dict['type'] == 'connected':
                    self.on_connected(data_dict['id'])
                else:
                    yield data
            elif opcode == websocket.ABNF.OPCODE_CLOSE:
                if data and len(data) >= 2:
                    code = 256 * six.byte2int(data[0:1]) + \
                        six.byte2int(data[1:2])
                    reason = data[2:].decode('utf-8')
                    self.on_close(code, reason)
                return
            else:
                yield ''



# -*- coding: utf-8 -*-
"""Unit tests for SentimentAnalysisClient"""

import pytest
from src.rev_ai.sentiment_analysis_client import SentimentAnalysisClient
from src.rev_ai import __version__
from src.rev_ai import Transcript, Monologue, Element, SentimentAnalysisJob, JobStatus, \
    SentimentAnalysisResult, SentimentValue, SentimentMessage, CustomerUrlData

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

TOKEN = 'token'
TEXT = 'Input'
JSON = Transcript([Monologue(0, [Element('text', 'hello', 0.0, 0.1, 100)])])
JOB_ID = '1'
METADATA = 'test'
NOTIFICATION_URL = 'https://example.com/'
NOTIFICATION_AUTH = 'headers'
CALLBACK_URL = 'https://example.com/'
CREATED_ON = '2018-05-05T23:23:22.29Z'
LANGUAGE = 'en'
SCORE = 1
CONTENT = 'random words'
OFFSET = 0
LENGTH = 12

NOTIFICATION_CONFIG = CustomerUrlData(NOTIFICATION_URL, NOTIFICATION_AUTH)


class TestSentimentAnalysisClient:
    def test_constructor_with_success(self):
        client = SentimentAnalysisClient(TOKEN)

        headers = client.default_headers

        assert headers.get('User-Agent') == 'RevAi-PythonSDK/{}'.format(__version__)
        assert headers.get('Authorization') == 'Bearer {}'.format(TOKEN)
        assert client.base_url == 'https://api.rev.ai/sentiment_analysis/v1/'

    @pytest.mark.parametrize('token', [None, ''])
    def test_constructor_with_no_token(self, token):
        with pytest.raises(ValueError, match='access_token must be provided'):
            SentimentAnalysisClient(token)

    def test_submit_job_text_with_success(self, mock_session, make_mock_response):
        client = SentimentAnalysisClient(TOKEN)
        url = urljoin(client.base_url, 'jobs')
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': CREATED_ON,
            'metadata': METADATA,
            'callback_url': CALLBACK_URL,
            'delete_after_seconds': 0,
            'language': LANGUAGE,
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.submit_job_from_text(text=TEXT,
                                          metadata=METADATA,
                                          callback_url=CALLBACK_URL,
                                          delete_after_seconds=0,
                                          language=LANGUAGE)

        assert res == SentimentAnalysisJob(JOB_ID,
                                           CREATED_ON,
                                           JobStatus.IN_PROGRESS,
                                           metadata=METADATA,
                                           callback_url=CALLBACK_URL,
                                           delete_after_seconds=0)
        mock_session.request.assert_called_once_with(
            "POST",
            url,
            json={
                'text': TEXT,
                'callback_url': CALLBACK_URL,
                'metadata': METADATA,
                'delete_after_seconds': 0,
                'language': LANGUAGE
            },
            headers=client.default_headers)

    def test_submit_job_text_auth_options_with_success(self, mock_session, make_mock_response):
        client = SentimentAnalysisClient(TOKEN)
        url = urljoin(client.base_url, 'jobs')
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': CREATED_ON,
            'metadata': METADATA,
            'delete_after_seconds': 0,
            'language': LANGUAGE,
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.submit_job_from_text(text=TEXT,
                                          metadata=METADATA,
                                          notification_config=NOTIFICATION_CONFIG,
                                          delete_after_seconds=0,
                                          language=LANGUAGE)

        assert res == SentimentAnalysisJob(JOB_ID,
                                           CREATED_ON,
                                           JobStatus.IN_PROGRESS,
                                           metadata=METADATA,
                                           delete_after_seconds=0)
        mock_session.request.assert_called_once_with(
            "POST",
            url,
            json={
                'text': TEXT,
                'notification_config': {'url': NOTIFICATION_URL, 'auth_headers': NOTIFICATION_AUTH},
                'metadata': METADATA,
                'delete_after_seconds': 0,
                'language': LANGUAGE
            },
            headers=client.default_headers)

    def test_submit_job_json_with_success(self, mock_session, make_mock_response):
        client = SentimentAnalysisClient(TOKEN)
        url = urljoin(client.base_url, 'jobs')
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': CREATED_ON,
            'metadata': METADATA,
            'callback_url': CALLBACK_URL,
            'delete_after_seconds': 0,
            'language': LANGUAGE,
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.submit_job_from_transcript(transcript=JSON,
                                                metadata=METADATA,
                                                callback_url=CALLBACK_URL,
                                                delete_after_seconds=0,
                                                language=LANGUAGE)

        assert res == SentimentAnalysisJob(JOB_ID,
                                           CREATED_ON,
                                           JobStatus.IN_PROGRESS,
                                           metadata=METADATA,
                                           callback_url=CALLBACK_URL,
                                           delete_after_seconds=0)
        mock_session.request.assert_called_once_with(
            "POST",
            url,
            json={
                'json': JSON.to_dict(),
                'callback_url': CALLBACK_URL,
                'metadata': METADATA,
                'delete_after_seconds': 0,
                'language': LANGUAGE
            },
            headers=client.default_headers)

    def test_submit_job_json_auth_options_with_success(self, mock_session, make_mock_response):
        client = SentimentAnalysisClient(TOKEN)
        url = urljoin(client.base_url, 'jobs')
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': CREATED_ON,
            'metadata': METADATA,
            'delete_after_seconds': 0,
            'language': LANGUAGE,
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.submit_job_from_transcript(transcript=JSON,
                                                metadata=METADATA,
                                                notification_config=NOTIFICATION_CONFIG,
                                                delete_after_seconds=0,
                                                language=LANGUAGE)

        assert res == SentimentAnalysisJob(JOB_ID,
                                           CREATED_ON,
                                           JobStatus.IN_PROGRESS,
                                           metadata=METADATA,
                                           delete_after_seconds=0)
        mock_session.request.assert_called_once_with(
            "POST",
            url,
            json={
                'json': JSON.to_dict(),
                'notification_config': {'url': NOTIFICATION_URL, 'auth_headers': NOTIFICATION_AUTH},
                'metadata': METADATA,
                'delete_after_seconds': 0,
                'language': LANGUAGE
            },
            headers=client.default_headers)

    def test_get_result_json_with_success(self, mock_session, make_mock_response):
        client = SentimentAnalysisClient(TOKEN)
        url = urljoin(client.base_url, 'jobs/{}/result?'.format(JOB_ID))
        data = {
            'messages': [
                {
                    'content': CONTENT,
                    'score': SCORE,
                    'sentiment': str(SentimentValue.NEGATIVE),
                    'offset': OFFSET,
                    'length': LENGTH
                }
            ]
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.get_result_json(JOB_ID)

        assert res == data
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=client.default_headers)

    def test_get_result_json_with_filter_for_with_success(self, mock_session, make_mock_response):
        client = SentimentAnalysisClient(TOKEN)
        url = urljoin(client.base_url, 'jobs/{0}/result?filter_for={1}'
                      .format(JOB_ID, SentimentValue.NEGATIVE))
        data = {
            'messages': [
                {
                    'content': CONTENT,
                    'score': SCORE,
                    'sentiment': str(SentimentValue.NEGATIVE),
                    'offset': OFFSET,
                    'length': LENGTH
                }
            ]
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.get_result_json(JOB_ID, SentimentValue.NEGATIVE)

        assert res == data
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=client.default_headers)

    def test_get_result_object_with_success(self, mock_session, make_mock_response):
        client = SentimentAnalysisClient(TOKEN)
        url = urljoin(client.base_url, 'jobs/{}/result?'.format(JOB_ID))
        data = {
            'messages': [
                {
                    'content': CONTENT,
                    'score': SCORE,
                    'sentiment': str(SentimentValue.NEGATIVE),
                    'offset': OFFSET,
                    'length': LENGTH
                }
            ]
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.get_result_object(JOB_ID)

        assert res == SentimentAnalysisResult(
            [SentimentMessage(
                CONTENT,
                SCORE,
                SentimentValue.NEGATIVE,
                offset=OFFSET,
                length=LENGTH
            )]
        )
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=client.default_headers)

    def test_get_result_object_with_filter_for_with_success(self, mock_session, make_mock_response):
        client = SentimentAnalysisClient(TOKEN)
        url = urljoin(client.base_url, 'jobs/{0}/result?filter_for={1}'
                      .format(JOB_ID, SentimentValue.POSITIVE))
        data = {
            'messages': [
                {
                    'content': CONTENT,
                    'score': SCORE,
                    'sentiment': str(SentimentValue.POSITIVE),
                    'offset': OFFSET,
                    'length': LENGTH
                }
            ]
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.get_result_object(JOB_ID, SentimentValue.POSITIVE)

        assert res == SentimentAnalysisResult(
            [SentimentMessage(
                CONTENT,
                SCORE,
                SentimentValue.POSITIVE,
                offset=OFFSET,
                length=LENGTH
            )]
        )
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=client.default_headers)



# -*- coding: utf-8 -*-
"""Unit tests for TopicExtractionClient"""

import pytest

from src.rev_ai.topic_extraction_client import TopicExtractionClient
from src.rev_ai import __version__
from src.rev_ai import Transcript, Monologue, Element, TopicExtractionJob, JobStatus, \
    TopicExtractionResult, Topic, Informant, CustomerUrlData

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

TOKEN = 'token'
TEXT = 'Input'
JSON = Transcript([Monologue(0, [Element('text', 'hello', 0.0, 0.1, 100)])])
JOB_ID = '1'
METADATA = 'test'
NOTIFICATION_URL = 'https://example.com/'
NOTIFICATION_AUTH = 'headers'
CREATED_ON = '2018-05-05T23:23:22.29Z'
LANGUAGE = 'en'
TOPIC_NAME = 'random'
SCORE = 1
INFORMANT_CONTENT = 'random words'
INFORMANT_OFFSET = 0
INFORMANT_LENGTH = 12
THRESHOLD = .3

NOTIFICATION_CONFIG = CustomerUrlData(NOTIFICATION_URL, NOTIFICATION_AUTH)


class TestTopicExtractionClient:
    def test_constructor_with_success(self):
        client = TopicExtractionClient(TOKEN)

        headers = client.default_headers

        assert headers.get('User-Agent') == 'RevAi-PythonSDK/{}'.format(__version__)
        assert headers.get('Authorization') == 'Bearer {}'.format(TOKEN)
        assert client.base_url == 'https://api.rev.ai/topic_extraction/v1/'

    @pytest.mark.parametrize('token', [None, ''])
    def test_constructor_with_no_token(self, token):
        with pytest.raises(ValueError, match='access_token must be provided'):
            TopicExtractionClient(token)

    def test_submit_job_text_with_success(self, mock_session, make_mock_response):
        client = TopicExtractionClient(TOKEN)
        url = urljoin(client.base_url, 'jobs')
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': CREATED_ON,
            'metadata': METADATA,
            'delete_after_seconds': 0,
            'language': LANGUAGE,
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.submit_job_from_text(text=TEXT,
                                          metadata=METADATA,
                                          callback_url=NOTIFICATION_URL,
                                          delete_after_seconds=0,
                                          language=LANGUAGE)

        assert res == TopicExtractionJob(JOB_ID,
                                         CREATED_ON,
                                         JobStatus.IN_PROGRESS,
                                         metadata=METADATA,
                                         delete_after_seconds=0)
        mock_session.request.assert_called_once_with(
            "POST",
            url,
            json={
                'text': TEXT,
                'callback_url': NOTIFICATION_URL,
                'metadata': METADATA,
                'delete_after_seconds': 0,
                'language': LANGUAGE
            },
            headers=client.default_headers)

    def test_submit_job_text_auth_options_with_success(self, mock_session, make_mock_response):
        client = TopicExtractionClient(TOKEN)
        url = urljoin(client.base_url, 'jobs')
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': CREATED_ON,
            'metadata': METADATA,
            'delete_after_seconds': 0,
            'language': LANGUAGE,
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.submit_job_from_text(text=TEXT,
                                          metadata=METADATA,
                                          notification_config=NOTIFICATION_CONFIG,
                                          delete_after_seconds=0,
                                          language=LANGUAGE)

        assert res == TopicExtractionJob(JOB_ID,
                                         CREATED_ON,
                                         JobStatus.IN_PROGRESS,
                                         metadata=METADATA,
                                         delete_after_seconds=0)
        mock_session.request.assert_called_once_with(
            "POST",
            url,
            json={
                'text': TEXT,
                'notification_config': {'url': NOTIFICATION_URL, 'auth_headers': NOTIFICATION_AUTH},
                'metadata': METADATA,
                'delete_after_seconds': 0,
                'language': LANGUAGE
            },
            headers=client.default_headers)

    def test_submit_job_json_with_success(self, mock_session, make_mock_response):
        client = TopicExtractionClient(TOKEN)
        url = urljoin(client.base_url, 'jobs')
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': CREATED_ON,
            'metadata': METADATA,
            'delete_after_seconds': 0,
            'language': LANGUAGE,
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.submit_job_from_transcript(transcript=JSON,
                                                metadata=METADATA,
                                                callback_url=NOTIFICATION_URL,
                                                delete_after_seconds=0,
                                                language=LANGUAGE)

        assert res == TopicExtractionJob(JOB_ID,
                                         CREATED_ON,
                                         JobStatus.IN_PROGRESS,
                                         metadata=METADATA,
                                         delete_after_seconds=0)
        mock_session.request.assert_called_once_with(
            "POST",
            url,
            json={
                'json': JSON.to_dict(),
                'callback_url': NOTIFICATION_URL,
                'metadata': METADATA,
                'delete_after_seconds': 0,
                'language': LANGUAGE
            },
            headers=client.default_headers)

    def test_submit_job_json_auth_options_with_success(self, mock_session, make_mock_response):
        client = TopicExtractionClient(TOKEN)
        url = urljoin(client.base_url, 'jobs')
        data = {
            'id': JOB_ID,
            'status': 'in_progress',
            'created_on': CREATED_ON,
            'metadata': METADATA,
            'delete_after_seconds': 0,
            'language': LANGUAGE,
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.submit_job_from_transcript(transcript=JSON,
                                                metadata=METADATA,
                                                notification_config=NOTIFICATION_CONFIG,
                                                delete_after_seconds=0,
                                                language=LANGUAGE)

        assert res == TopicExtractionJob(JOB_ID,
                                         CREATED_ON,
                                         JobStatus.IN_PROGRESS,
                                         metadata=METADATA,
                                         delete_after_seconds=0)
        mock_session.request.assert_called_once_with(
            "POST",
            url,
            json={
                'json': JSON.to_dict(),
                'notification_config': {'url': NOTIFICATION_URL, 'auth_headers': NOTIFICATION_AUTH},
                'metadata': METADATA,
                'delete_after_seconds': 0,
                'language': LANGUAGE
            },
            headers=client.default_headers)

    def test_get_result_json_with_success(self, mock_session, make_mock_response):
        client = TopicExtractionClient(TOKEN)
        url = urljoin(client.base_url, 'jobs/{}/result?'.format(JOB_ID))
        data = {
            'topics': [
                {
                    'topic_name': TOPIC_NAME,
                    'score': SCORE,
                    'informants': [
                        {
                            'content': INFORMANT_CONTENT,
                            'offset': INFORMANT_OFFSET,
                            'length': INFORMANT_LENGTH
                        }
                    ]
                }
            ]
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.get_result_json(JOB_ID)

        assert res == data
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=client.default_headers)

    def test_get_result_json_with_threshold_with_success(self, mock_session, make_mock_response):
        client = TopicExtractionClient(TOKEN)
        url = urljoin(client.base_url, 'jobs/{0}/result?threshold={1}'.format(JOB_ID, THRESHOLD))
        data = {
            'topics': [
                {
                    'topic_name': TOPIC_NAME,
                    'score': SCORE,
                    'informants': [
                        {
                            'content': INFORMANT_CONTENT,
                            'offset': INFORMANT_OFFSET,
                            'length': INFORMANT_LENGTH
                        }
                    ]
                }
            ]
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.get_result_json(JOB_ID, THRESHOLD)

        assert res == data
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=client.default_headers)

    def test_get_result_object_with_success(self, mock_session, make_mock_response):
        client = TopicExtractionClient(TOKEN)
        url = urljoin(client.base_url, 'jobs/{}/result?'.format(JOB_ID))
        data = {
            'topics': [
                {
                    'topic_name': TOPIC_NAME,
                    'score': SCORE,
                    'informants': [
                        {
                            'content': INFORMANT_CONTENT,
                            'offset': INFORMANT_OFFSET,
                            'length': INFORMANT_LENGTH
                        }
                    ]
                }
            ]
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.get_result_object(JOB_ID)

        assert res == TopicExtractionResult(
            [Topic(
                TOPIC_NAME,
                SCORE,
                [Informant(INFORMANT_CONTENT, offset=INFORMANT_OFFSET, length=INFORMANT_LENGTH)]
            )]
        )
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=client.default_headers)

    def test_get_result_object_with_threshold_with_success(self, mock_session, make_mock_response):
        client = TopicExtractionClient(TOKEN)
        url = urljoin(client.base_url, 'jobs/{0}/result?threshold={1}'.format(JOB_ID, THRESHOLD))
        data = {
            'topics': [
                {
                    'topic_name': TOPIC_NAME,
                    'score': SCORE,
                    'informants': [
                        {
                            'content': INFORMANT_CONTENT,
                            'offset': INFORMANT_OFFSET,
                            'length': INFORMANT_LENGTH
                        }
                    ]
                }
            ]
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.get_result_object(JOB_ID, THRESHOLD)

        assert res == TopicExtractionResult(
            [Topic(
                TOPIC_NAME,
                SCORE,
                [Informant(INFORMANT_CONTENT, offset=INFORMANT_OFFSET, length=INFORMANT_LENGTH)]
            )]
        )
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=client.default_headers)



# -*- coding: utf-8 -*-

import pytest
from src.rev_ai.streamingclient import RevAiStreamingClient
from src.rev_ai.models.streaming import MediaConfig


@pytest.fixture
def mock_streaming_client(mocker):
    config = MediaConfig()
    streaming_client = RevAiStreamingClient('token', config)
    streaming_client.client.connect = mocker.Mock(name="mock_connect")
    streaming_client.client.abort = mocker.Mock(name="mock_abort")
    streaming_client.client.send_binary = mocker.Mock(name="mock_send_binary")
    streaming_client.client.recv_data = mocker.Mock(name="mock_recv")
    streaming_client.client.send = mocker.Mock(name="mock_send")
    return streaming_client


@pytest.fixture
def mock_generator(mocker):
    def gen():
        for i in range(0, 3):
            yield i
    return gen



# -*- coding: utf-8 -*-
"""Unit tests for Language Identification Client"""

import json
import pytest
from src.rev_ai.language_identification_client import LanguageIdentificationClient
from src.rev_ai import __version__
from src.rev_ai import LanguageIdentificationJob, JobStatus, \
    LanguageIdentificationResult, LanguageConfidence

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

TOKEN = 'token'
JOB_ID = '1'
METADATA = 'test'
MEDIA_URL = 'https://www.rev.ai/FTC_Sample_1.mp3'
CALLBACK_URL = 'https://example.com/'
CREATED_ON = '2018-05-05T23:23:22.29Z'
FILENAME = 'test.mp3'
TOP_LANGUAGE = 'en'
SCORE = 1


class TestLanguageIdentificationClient:
    def test_constructor_with_success(self):
        client = LanguageIdentificationClient(TOKEN)

        headers = client.default_headers

        assert headers.get('User-Agent') == 'RevAi-PythonSDK/{}'.format(__version__)
        assert headers.get('Authorization') == 'Bearer {}'.format(TOKEN)
        assert client.base_url == 'https://api.rev.ai/languageid/v1/'

    @pytest.mark.parametrize('token', [None, ''])
    def test_constructor_with_no_token(self, token):
        with pytest.raises(ValueError, match='access_token must be provided'):
            LanguageIdentificationClient(token)

    def test_submit_job_url_with_success(self, mock_session, make_mock_response):
        client = LanguageIdentificationClient(TOKEN)
        url = urljoin(client.base_url, 'jobs')
        data = {
            'id': JOB_ID,
            'created_on': CREATED_ON,
            'media_url': MEDIA_URL,
            'callback_url': CALLBACK_URL,
            'metadata': METADATA,
            'status': 'in_progress',
            'delete_after_seconds': 0
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.submit_job_url(media_url=MEDIA_URL,
                                    metadata=METADATA,
                                    callback_url=CALLBACK_URL,
                                    delete_after_seconds=0)

        assert res == LanguageIdentificationJob(JOB_ID,
                                                CREATED_ON,
                                                JobStatus.IN_PROGRESS,
                                                metadata=METADATA,
                                                callback_url=CALLBACK_URL,
                                                media_url=MEDIA_URL,
                                                delete_after_seconds=0)
        mock_session.request.assert_called_once_with(
            "POST",
            url,
            json={
                'media_url': MEDIA_URL,
                'callback_url': CALLBACK_URL,
                'metadata': METADATA,
                'delete_after_seconds': 0
            },
            headers=client.default_headers)

    def test_submit_job_local_file_with_success(self, mocker, mock_session, make_mock_response):
        client = LanguageIdentificationClient(TOKEN)
        url = urljoin(client.base_url, 'jobs')
        data = {
            'id': JOB_ID,
            'created_on': CREATED_ON,
            'callback_url': CALLBACK_URL,
            'metadata': METADATA,
            'status': 'in_progress',
            'delete_after_seconds': 0
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        with mocker.patch('src.rev_ai.language_identification_client.open', create=True)() as file:
            res = client.submit_job_local_file(FILENAME, METADATA,
                                               CALLBACK_URL, 0)

            assert res == LanguageIdentificationJob(JOB_ID,
                                                    CREATED_ON,
                                                    JobStatus.IN_PROGRESS,
                                                    metadata=METADATA,
                                                    callback_url=CALLBACK_URL,
                                                    delete_after_seconds=0)
            mock_session.request.assert_called_once_with(
                "POST",
                url,
                files={
                    'media': (FILENAME, file),
                    'options': (
                        None,
                        json.dumps({
                            'metadata': METADATA,
                            'callback_url': CALLBACK_URL,
                            'delete_after_seconds': 0
                        }, sort_keys=True)
                    )
                },
                headers=client.default_headers)

    def test_get_result_json_with_success(self, mock_session, make_mock_response):
        client = LanguageIdentificationClient(TOKEN)
        url = urljoin(client.base_url, 'jobs/{}/result'.format(JOB_ID))
        data = {
            'top_language': TOP_LANGUAGE,
            'language_confidences': [
                {
                    "language": "en",
                    "confidence": 0.907
                },
                {
                    "language": "nl",
                    "confidence": 0.023
                },
                {
                    "language": "ar",
                    "confidence": 0.023
                },
                {
                    "language": "de",
                    "confidence": 0.023
                },
                {
                    "language": "cmn",
                    "confidence": 0.023
                }
            ]
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.get_result_json(JOB_ID)

        assert res == data
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=client.default_headers)

    def test_get_result_object_with_success(self, mock_session, make_mock_response):
        client = LanguageIdentificationClient(TOKEN)
        url = urljoin(client.base_url, 'jobs/{}/result'.format(JOB_ID))
        data = {
            'top_language': TOP_LANGUAGE,
            'language_confidences': [
                {
                    "language": "en",
                    "confidence": 0.907
                },
                {
                    "language": "nl",
                    "confidence": 0.023
                },
                {
                    "language": "ar",
                    "confidence": 0.023
                },
                {
                    "language": "de",
                    "confidence": 0.023
                },
                {
                    "language": "cmn",
                    "confidence": 0.023
                }
            ]
        }
        response = make_mock_response(url=url, json_data=data)
        mock_session.request.return_value = response

        res = client.get_result_object(JOB_ID)

        assert res == LanguageIdentificationResult(
            top_language=TOP_LANGUAGE,
            language_confidences=[
                LanguageConfidence(
                    language="en",
                    confidence=.907
                ),
                LanguageConfidence(
                    language="nl",
                    confidence=.023
                ),
                LanguageConfidence(
                    language="ar",
                    confidence=.023
                ),
                LanguageConfidence(
                    language="de",
                    confidence=.023
                ),
                LanguageConfidence(
                    language="cmn",
                    confidence=.023
                )
            ])
        mock_session.request.assert_called_once_with(
            "GET",
            url,
            headers=client.default_headers)



# -*- coding: utf-8 -*-
"""Mock RevAiAPIClient for testing purposes"""

import pytest
import json
import requests
from src.rev_ai.apiclient import RevAiAPIClient


@pytest.fixture
def mock_session(mocker):
    mock_session = mocker.patch.object(requests, 'Session', autospec=True)
    mock_session.return_value.__enter__.return_value = mock_session
    return mock_session


@pytest.fixture
def make_mock_response(mocker):
    def _mock_response(url="", status=200, json_data=None, text=""):
        response = requests.Response()
        response.status_code = status
        response.reason = 'Testing'
        response.url = url
        if text:
            type(response).text = mocker.PropertyMock(return_value=text)
            type(response).content = mocker.PropertyMock(return_value=text)
        if json_data:
            response.json = mocker.Mock(return_value=json_data)
            type(response).content = mocker.PropertyMock(
                return_value=str(json_data).encode('utf-8'))
        return response
    return _mock_response
